library ieee;
use ieee.std_logic_1164.all;

entity mux4a1 is
  port (enable: in std_logic;
        x: in std_logic_vector(3 downto 0);
        sel: in std_logic_vector(1 downto 0);
        y: out std_logic);
end mux4a1;

architecture puertas of mux4a1 is

signal not_sel1, not_sel0: std_logic;
signal s0, s1, s2, s3: std_logic;

begin
  
not1: entity work.not1 port map(sel(0), not_sel0);
not2: entity work.not1 port map(sel(1), not_sel1);
and1: entity work.and4 port map(enable, not_sel0, not_sel1, x(0), s0);
and2: entity work.and4 port map(enable, sel(0), not_sel1, x(1), s1);
and3: entity work.and4 port map(enable, not_sel0, sel(1), x(2), s2);
and4: entity work.and4 port map(enable, sel(0), sel(1), x(3), s3);
or1:  entity work.or4  port map(s0, s1, s2, s3, y);

end puertas;

architecture concurrente of mux4a1 is
begin
  y <= enable and (
       ( (not sel(0)) and (not sel(1)) and x(0) ) or
       ( sel(0) and (not sel(1)) and x(1) ) or
       ( (not sel(0)) and sel(1) and x(2) ) or
       ( sel(0) and sel(1) and x(3) )
       );
end concurrente;

architecture funcional of mux4a1 is
begin
  process(enable, x, sel)
    begin
      if enable = '0' then
        y <= '0';
      else
        case sel is
          when "00" => y <= x(0);
          when "01" => y <= x(1);
          when "10" => y <= x(2);
          when others => y <= x(3);
        end case;
      end if;        
    end process;
end funcional;
-------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity test_mux4a1 is
end test_mux4a1;

architecture test of test_mux4a1 is

signal enable, y: std_logic;
signal x: std_logic_vector(3 downto 0) := "0000";
signal sel: std_logic_vector(1 downto 0);

begin
  
  inst1: entity work.mux4a1(funcional) port map(enable, x, sel, y);
  
  enable <= '1', '0' after 400 ns;
  x(0) <= not x(0) after 2 ns;
  x(1) <= not x(1) after 4 ns;
  x(2) <= not x(2) after 8 ns;
  x(3) <= not x(3) after 16 ns;
  sel <= "00", "01" after 100 ns, "10" after 200 ns, "11" after 300 ns;
  
end test;